@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2016 Dmitry Kazakov <dimula73@gmail.com>
+ *  SPDX-FileCopyrightText: 2016 Dmitry Kazakov <dimula73@gmail.com>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef KOSHAPERESIZECOMMAND_H
@@ -45,11 +33,17 @@ public:
     void redoImpl() override;
     void undoImpl() override;
 
+    QMap<KoShape*, QRectF> redoNoUpdate();
+    QMap<KoShape*, QRectF> undoNoUpdate();
+
     int id() const override;
     bool mergeWith(const KUndo2Command *command) override;
 
+    void replaceResizeAction(qreal scaleX, qreal scaleY,
+                             const QPointF &absoluteStillPoint);
+
 private:
-    class Private;
+    struct Private;
     QScopedPointer<Private> const m_d;
 
 };
