@@ -1,19 +1,7 @@
 /*
- *  Copyright (c) 2009 Boudewijn Rempt <boud@valdyas.org>
+ *  SPDX-FileCopyrightText: 2009 Boudewijn Rempt <boud@valdyas.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  SPDX-License-Identifier: GPL-2.0-or-later
  */
 #include "psd_loader.h"
 
@@ -38,9 +26,11 @@
 #include <kis_paint_device.h>
 #include <kis_transaction.h>
 #include <kis_transparency_mask.h>
+#include <kis_generator_layer.h>
+#include <kis_generator_registry.h>
 
 #include <kis_asl_layer_style_serializer.h>
-#include <kis_psd_layer_style_resource.h>
+#include <asl/kis_asl_xml_parser.h>
 #include "KisResourceServerProvider.h"
 
 #include "psd.h"
@@ -51,11 +41,16 @@
 #include "psd_layer_section.h"
 #include "psd_resource_block.h"
 #include "psd_image_data.h"
+#include "KisEmbeddedResourceStorageProxy.h"
+#include "KisImageBarrierLock.h"
+#include "KisImportUserFeedbackInterface.h"
 
-PSDLoader::PSDLoader(KisDocument *doc)
+
+PSDLoader::PSDLoader(KisDocument *doc, KisImportUserFeedbackInterface *feedbackInterface)
     : m_image(0)
     , m_doc(doc)
     , m_stop(false)
+    , m_feedbackInterface(feedbackInterface)
 {
 }
 
@@ -63,11 +58,11 @@ PSDLoader::~PSDLoader()
 {
 }
 
-KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
+KisImportExportErrorCode PSDLoader::decode(QIODevice &io)
 {
     // open the file
 
-    dbgFile << "pos:" << io->pos();
+    dbgFile << "pos:" << io.pos();
 
     PSDHeader header;
     if (!header.read(io)) {
@@ -76,7 +71,7 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
     }
 
     dbgFile << header;
-    dbgFile << "Read header. pos:" << io->pos();
+    dbgFile << "Read header. pos:" << io.pos();
 
     PSDColorModeBlock colorModeBlock(header.colormode);
     if (!colorModeBlock.read(io)) {
@@ -84,21 +79,21 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
         return ImportExportCodes::FileFormatIncorrect;
     }
 
-    dbgFile << "Read color mode block. pos:" << io->pos();
+    dbgFile << "Read color mode block. pos:" << io.pos();
 
     PSDImageResourceSection resourceSection;
     if (!resourceSection.read(io)) {
         dbgFile << "failed image reading resource section: " << resourceSection.error;
         return ImportExportCodes::FileFormatIncorrect;
     }
-    dbgFile << "Read image resource section. pos:" << io->pos();
+    dbgFile << "Read image resource section. pos:" << io.pos();
 
     PSDLayerMaskSection layerSection(header);
     if (!layerSection.read(io)) {
         dbgFile << "failed reading layer/mask section: " << layerSection.error;
         return ImportExportCodes::FileFormatIncorrect;
     }
-    dbgFile << "Read layer/mask section. " << layerSection.nLayers << "layers. pos:" << io->pos();
+    dbgFile << "Read layer/mask section. " << layerSection.nLayers << "layers. pos:" << io.pos();
 
     // Done reading, except possibly for the image data block, which is only relevant if there
     // are no layers.
@@ -131,11 +126,12 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
     }
 
     // Creating the KisImage
-    QFile *file = dynamic_cast<QFile*>(io);
+    QFile *file = dynamic_cast<QFile *>(&io);
     QString name = file ? file->fileName() : "Imported";
     m_image = new KisImage(m_doc->createUndoStore(),  header.width, header.height, cs, name);
     Q_CHECK_PTR(m_image);
-    m_image->lock();
+
+    KisImageBarrierLock lock(m_image);
 
     // set the correct resolution
     if (resourceSection.resources.contains(PSDImageResourceSection::RESN_INFO)) {
@@ -162,15 +158,39 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
         m_image->addAnnotation(annotation);
     }
 
+    // Load embedded patterns early for fill layers.
+
+    const QVector<QDomDocument> &embeddedPatterns =
+        layerSection.globalInfoSection.embeddedPatterns;
+
+    const QString storageLocation = m_doc->embeddedResourcesStorageId();
+
+    KisEmbeddedResourceStorageProxy resourceProxy(storageLocation);
+
+    KisAslLayerStyleSerializer serializer;
+    if (!embeddedPatterns.isEmpty()) {
+        Q_FOREACH (const QDomDocument &doc, embeddedPatterns) {
+            serializer.registerPSDPattern(doc);
+        }
+        Q_FOREACH (KoPatternSP pattern, serializer.patterns()) {
+            if (pattern && pattern->valid()) {
+                resourceProxy.addResource(pattern);
+                dbgFile << "Loaded embedded pattern: " << pattern->name();
+            }
+            else {
+                qWarning() << "Invalid or empty pattern" << pattern;
+            }
+        }
+    }
 
     // Read the projection into our single layer. Since we only read the projection when
     // we have just one layer, we don't need to later on apply the alpha channel of the
     // first layer to the projection if the number of layers is negative/
     // See https://www.adobe.com/devnet-apps/photoshop/fileformatashtml/#50577409_16000.
     if (layerSection.nLayers == 0) {
-        dbgFile << "Position" << io->pos() << "Going to read the projection into the first layer, which Photoshop calls 'Background'";
+        dbgFile << "Position" << io.pos() << "Going to read the projection into the first layer, which Photoshop calls 'Background'";
 
-        KisPaintLayerSP layer = new KisPaintLayer(m_image, i18n("Background"), OPACITY_OPAQUE_U8);
+        KisPaintLayerSP layer = new KisPaintLayer(m_image, i18nc("Name for the bottom-most layer in the layerstack", "Background"), OPACITY_OPAQUE_U8);
 
         PSDImageData imageData(&header);
         imageData.read(io, layer->paintDevice());
@@ -178,7 +198,6 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
         m_image->addNode(layer, m_image->rootLayer());
 
         // Only one layer, the background layer, so we're done.
-        m_image->unlock();
         return ImportExportCodes::OK;
     }
 
@@ -197,6 +216,7 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
 
     typedef QPair<QDomDocument, KisLayerSP> LayerStyleMapping;
     QVector<LayerStyleMapping> allStylesXml;
+    using namespace std::placeholders;
 
     // read the channels for the various layers
     for(int i = 0; i < layerSection.nLayers; ++i) {
@@ -269,21 +289,78 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
                           << "    " << ppVar(groupStack.size());
                 continue;
             }
-        }
-        else {
-            KisPaintLayerSP layer = new KisPaintLayer(m_image, layerRecord->layerName, layerRecord->opacity);
+        } else {
+            KisLayerSP layer;
+            if (!layerRecord->infoBlocks.fillConfig.isNull()) {
+                KisFilterConfigurationSP cfg;
+                QDomDocument fillConfig;
+                KisAslCallbackObjectCatcher catcher;
+                if (layerRecord->infoBlocks.fillType == psd_fill_gradient) {
+                    cfg = KisGeneratorRegistry::instance()->value("gradient")->defaultConfiguration(resourceProxy.resourcesInterface());
+
+                    psd_layer_gradient_fill fill;
+                    fill.imageWidth = m_image->width();
+                    fill.imageHeight = m_image->height();
+                    catcher.subscribeGradient("/null/Grad", std::bind(&psd_layer_gradient_fill::setGradient, &fill, _1));
+                    catcher.subscribeBoolean("/null/Dthr", std::bind(&psd_layer_gradient_fill::setDither, &fill, _1));
+                    catcher.subscribeBoolean("/null/Rvrs", std::bind(&psd_layer_gradient_fill::setReverse, &fill, _1));
+                    catcher.subscribeUnitFloat("/null/Angl", "#Ang", std::bind(&psd_layer_gradient_fill::setAngle, &fill, _1));
+                    catcher.subscribeEnum("/null/Type", "GrdT", std::bind(&psd_layer_gradient_fill::setType, &fill, _1));
+                    catcher.subscribeBoolean("/null/Algn", std::bind(&psd_layer_gradient_fill::setAlignWithLayer, &fill, _1));
+                    catcher.subscribeUnitFloat("/null/Scl ", "#Prc", std::bind(&psd_layer_gradient_fill::setScale, &fill, _1));
+                    catcher.subscribePoint("/null/Ofst", std::bind(&psd_layer_gradient_fill::setOffset, &fill, _1));
+                    KisAslXmlParser parser;
+                    parser.parseXML(layerRecord->infoBlocks.fillConfig, catcher);
+                    fillConfig = fill.getFillLayerConfig();
+
+                } else if (layerRecord->infoBlocks.fillType == psd_fill_pattern) {
+                    cfg = KisGeneratorRegistry::instance()->value("pattern")->defaultConfiguration(resourceProxy.resourcesInterface());
+
+                    psd_layer_pattern_fill fill;
+                    catcher.subscribeUnitFloat("/null/Angl", "#Ang", std::bind(&psd_layer_pattern_fill::setAngle, &fill, _1));
+                    catcher.subscribeUnitFloat("/null/Scl ", "#Prc", std::bind(&psd_layer_pattern_fill::setScale, &fill, _1));
+                    catcher.subscribeBoolean("/null/Algn", std::bind(&psd_layer_pattern_fill::setAlignWithLayer, &fill, _1));
+                    catcher.subscribePoint("/null/phase", std::bind(&psd_layer_pattern_fill::setOffset, &fill, _1));
+                    catcher.subscribePatternRef("/null/Ptrn", std::bind(&psd_layer_pattern_fill::setPatternRef, &fill, _1, _2));
+
+                    KisAslXmlParser parser;
+                    parser.parseXML(layerRecord->infoBlocks.fillConfig, catcher);
+                    fillConfig = fill.getFillLayerConfig();
+
+                } else {
+                    cfg = KisGeneratorRegistry::instance()->value("color")->defaultConfiguration(resourceProxy.resourcesInterface());
+
+                    psd_layer_solid_color fill;
+                    fill.cs = m_image->colorSpace();
+                    catcher.subscribeColor("/null/Clr ", std::bind(&psd_layer_solid_color::setColor, &fill, _1));
+                    KisAslXmlParser parser;
+                    parser.parseXML(layerRecord->infoBlocks.fillConfig, catcher);
+
+                    fillConfig = fill.getFillLayerConfig();
+                }
+                cfg->fromXML(fillConfig.firstChildElement());
+                cfg->createLocalResourcesSnapshot();
+                KisGeneratorLayerSP genlayer = new KisGeneratorLayer(m_image, layerRecord->layerName, cfg, m_image->globalSelection());
+                genlayer->setFilter(cfg);
+                layer = genlayer;
+
+            } else {
+                layer = new KisPaintLayer(m_image, layerRecord->layerName, layerRecord->opacity);
+                if (!layerRecord->readPixelData(io, layer->paintDevice())) {
+                    dbgFile << "failed reading channels for layer: " << layerRecord->layerName << layerRecord->error;
+                    return ImportExportCodes::FileFormatIncorrect;
+                }
+            }
             layer->setCompositeOpId(psd_blendmode_to_composite_op(layerRecord->blendModeKey));
 
+            layer->setColorLabelIndex(layerRecord->labelColor);
+
             const QDomDocument &styleXml = layerRecord->infoBlocks.layerStyleXml;
 
             if (!styleXml.isNull()) {
                 allStylesXml << LayerStyleMapping(styleXml, layer);
             }
 
-            if (!layerRecord->readPixelData(io, layer->paintDevice())) {
-                dbgFile << "failed reading channels for layer: " << layerRecord->layerName << layerRecord->error;
-                return ImportExportCodes::FileFormatIncorrect;
-            }
             if (!groupStack.isEmpty()) {
                 m_image->addNode(layer, groupStack.top());
             }
@@ -297,44 +374,61 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
 
         Q_FOREACH (ChannelInfo *channelInfo, layerRecord->channelInfoRecords) {
             if (channelInfo->channelId < -1) {
-                KisTransparencyMaskSP mask = new KisTransparencyMask();
-                mask->setName(i18n("Transparency Mask"));
-                mask->initSelection(newLayer);
-                if (!layerRecord->readMask(io, mask->paintDevice(), channelInfo)) {
-                    dbgFile << "failed reading masks for layer: " << layerRecord->layerName << layerRecord->error;
+                const KisGeneratorLayer *fillLayer = qobject_cast<KisGeneratorLayer *>(newLayer.data());
+                if (fillLayer) {
+                    if (!layerRecord->readMask(io, fillLayer->paintDevice(), channelInfo)) {
+                        dbgFile << "failed reading masks for generator layer: " << layerRecord->layerName << layerRecord->error;
+                    }
+                } else {
+                    KisTransparencyMaskSP mask = new KisTransparencyMask(m_image, i18n("Transparency Mask"));
+                    mask->initSelection(newLayer);
+                    if (!layerRecord->readMask(io, mask->paintDevice(), channelInfo)) {
+                        dbgFile << "failed reading masks for layer: " << layerRecord->layerName << layerRecord->error;
+                    }
+                    m_image->addNode(mask, newLayer);
                 }
-                m_image->addNode(mask, newLayer);
             }
         }
 
         lastAddedLayer = newLayer;
     }
 
-    const QVector<QDomDocument> &embeddedPatterns =
-        layerSection.globalInfoSection.embeddedPatterns;
-
-    KisAslLayerStyleSerializer serializer;
-
-    if (!embeddedPatterns.isEmpty()) {
-        Q_FOREACH (const QDomDocument &doc, embeddedPatterns) {
-            serializer.registerPSDPattern(doc);
-        }
-    }
-
-    QVector<KisPSDLayerStyleSP> allStylesForServer;
-
     if (!allStylesXml.isEmpty()) {
         Q_FOREACH (const LayerStyleMapping &mapping, allStylesXml) {
+
             serializer.readFromPSDXML(mapping.first);
 
             if (serializer.styles().size() == 1) {
                 KisPSDLayerStyleSP layerStyle = serializer.styles().first();
                 KisLayerSP layer = mapping.second;
 
-                layerStyle->setName(layer->name());
+                Q_FOREACH (KoAbstractGradientSP gradient, serializer.gradients()) {
+                    if (gradient && gradient->valid()) {
+                        resourceProxy.addResource(gradient);
+                    }
+                    else {
+                        qWarning() << "Invalid or empty gradient" << gradient;
+                    }
+                }
 
-                allStylesForServer << layerStyle;
-                layer->setLayerStyle(layerStyle->clone());
+                Q_FOREACH (const KoPatternSP &pattern, serializer.patterns()) {
+                    if (pattern && pattern->valid()) {
+                        resourceProxy.addResource(pattern);
+                    } else {
+                        qWarning() << "Invalid or empty pattern" << pattern;
+                    }
+                }
+
+                layerStyle->setName(layer->name());
+                layerStyle->setResourcesInterface(resourceProxy.detachedResourcesInterface());
+                if (!layerStyle->uuid().isNull()) {
+                    layerStyle->setUuid(QUuid::createUuid());
+                }
+                layerStyle->setValid(true);
+
+                resourceProxy.addResource(layerStyle);
+
+                layer->setLayerStyle(layerStyle->cloneWithResourcesSnapshot(layerStyle->resourcesInterface(), 0));
             } else {
                 warnKrita << "WARNING: Couldn't read layer style!" << ppVar(serializer.styles());
             }
@@ -342,26 +436,10 @@ KisImportExportErrorCode PSDLoader::decode(QIODevice *io)
         }
     }
 
-    if (!allStylesForServer.isEmpty()) {
-        KisPSDLayerStyleCollectionResource *collection =
-            new KisPSDLayerStyleCollectionResource("Embedded PSD Styles.asl");
-
-        collection->setName(i18nc("Auto-generated layer style collection name for embedded styles (collection)", "<%1> (embedded)", m_image->objectName()));
-        KIS_SAFE_ASSERT_RECOVER_NOOP(!collection->valid());
-
-        collection->setLayerStyles(allStylesForServer);
-        KIS_SAFE_ASSERT_RECOVER_NOOP(collection->valid());
-
-        KoResourceServer<KisPSDLayerStyleCollectionResource> *server = KisResourceServerProvider::instance()->layerStyleCollectionServer();
-        server->addResource(collection, false);
-    }
-
-
-    m_image->unlock();
-    return ImportExportCodes::OK;
+    return KritaUtils::workaroundUnsuitableImageColorSpace(m_image, m_feedbackInterface, lock);
 }
 
-KisImportExportErrorCode PSDLoader::buildImage(QIODevice *io)
+KisImportExportErrorCode PSDLoader::buildImage(QIODevice &io)
 {
     return decode(io);
 }
